@@ -568,6 +568,12 @@ static int identify_new(struct fet_device *dev, const char *force_id)
 		return -1;
 	}
 
+	/* This packet seems to be necessary in order to program on the
+	 * MSP430FR5739 development board.
+	 */
+	if (xfer(dev, 0x30, NULL, 0, 0) < 0)
+		printc_dbg("fet: warning: message 0x30 failed\n");
+
 	return 0;
 }
 
@@ -1047,7 +1053,7 @@ static device_t fet_open_rf2500(const struct device_args *args)
 		return NULL;
 	}
 
-        trans = rf2500_open(args->path);
+        trans = rf2500_open(args->path, args->requested_serial);
         if (!trans)
                 return NULL;
 
@@ -1076,7 +1082,7 @@ static device_t fet_open_olimex(const struct device_args *args)
 	if (args->flags & DEVICE_FLAG_TTY)
 		trans = uif_open(args->path, UIF_TYPE_OLIMEX);
 	else
-		trans = olimex_open(args->path);
+		trans = olimex_open(args->path, args->requested_serial);
 
         if (!trans)
                 return NULL;
